@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	3 October 1995
- * Last Edited:	6 January 1999
+ * Last Edited:	21 September 1999
  *
  * Copyright 1999 by the University of Washington
  *
@@ -235,7 +235,8 @@ long mbx_create (MAILSTREAM *stream,char *mailbox)
     return NIL;
   }
   memset (tmp,'\0',HDRSIZE);	/* initialize header */
-  sprintf (s = tmp,"*mbx*\015\012%08lx00000000\015\012",time (0));
+  sprintf (s = tmp,"*mbx*\015\012%08lx00000000\015\012",
+	   (unsigned long) time (0));
   s += strlen (s);		/* move to end of pointer */
   for (i = 0; i < NUSERFLAGS; ++i) *s++ ='\015',*s++ = '\012';
   if (write (fd,tmp,HDRSIZE) != HDRSIZE) {
@@ -294,19 +295,24 @@ long mbx_rename (MAILSTREAM *stream,char *old,char *newname)
     return NIL;
   }
   if (newname) {		/* want rename? */
-    if (!((s = mbx_file (tmp,newname)) && *s)) {
+    if (!((s = mailboxfile (tmp,newname)) && *s) ||
+	((s = strrchr (s,'\\')) && !s[1])) {
       sprintf (tmp,"Can't rename mailbox %.80s to %.80s: invalid name",
 	       old,newname);
       mm_log (tmp,ERROR);
       ret = NIL;		/* set failure */
     }
-    if (s = strrchr (s,'\\')) {	/* found superior to destination name? */
-      c = *++s;			/* remember first character of inferior */
-      *s = '\0';		/* tie off to get just superior */
+				/* found superior to destination name? */
+    if (s && (s != tmp) && ((tmp[1] != ':') || (s != tmp + 2))) {
+      c = s[1];			/* remember character after delimiter */
+      *s = s[1] = '\0';		/* tie off name at delimiter */
 				/* name doesn't exist, create it */
-      if ((stat (tmp,&sbuf) || ((sbuf.st_mode & S_IFMT) != S_IFDIR)) &&
-	  !dummy_create (stream,tmp)) ret = NIL;
-      else *s = c;		/* restore full name */
+      if (stat (tmp,&sbuf) || ((sbuf.st_mode & S_IFMT) != S_IFDIR)) {
+	*s = '\\';		/* restore delimiter */
+	if (!dummy_create (stream,tmp)) ret = NIL;
+      }
+      else *s = '\\';		/* restore delimiter */
+      s[1] = c;			/* restore character after delimiter */
     }
     flock (fd,LOCK_UN);		/* release lock on the file */
     close (fd);			/* pacify NTFS */
@@ -362,6 +368,8 @@ MAILSTREAM *mbx_open (MAILSTREAM *stream)
   stream->local = memset (fs_get (sizeof (MBXLOCAL)),NIL,sizeof (MBXLOCAL));
   LOCAL->buf = (char *) fs_get (MAXMESSAGESIZE + 1);
   LOCAL->buflen = MAXMESSAGESIZE;
+				/* note if an INBOX or not */
+  stream->inbox = !strcmp(ucase (strcpy (LOCAL->buf,stream->mailbox)),"INBOX");
   fs_give ((void **) &stream->mailbox);
   stream->mailbox = cpystr (tmp);
 				/* get parse/append permission */
@@ -770,9 +778,10 @@ long mbx_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 	     elt->private.special.text.size,L_SET);
       mail_date(LOCAL->buf,elt);/* build target header */
       sprintf (LOCAL->buf+strlen(LOCAL->buf),",%lu;%08lx%04x-00000000\015\012",
-	       elt->rfc822_size,elt->user_flags,(fSEEN * elt->seen) +
-	       (fDELETED * elt->deleted) + (fFLAGGED * elt->flagged) +
-	       (fANSWERED * elt->answered) + (fDRAFT * elt->draft));
+	       elt->rfc822_size,elt->user_flags,(unsigned)
+	       ((fSEEN * elt->seen) + (fDELETED * elt->deleted) +
+		(fFLAGGED * elt->flagged) + (fANSWERED * elt->answered) +
+		(fDRAFT * elt->draft)));
 				/* write target header */
       if (ret = (write (fd,LOCAL->buf,strlen (LOCAL->buf)) > 0))
 	for (k = elt->rfc822_size; ret && (j = min (k,LOCAL->buflen)); k -= j){
@@ -886,7 +895,8 @@ long mbx_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
   if (date) mail_date(tmp,&elt);/* write preseved date */
   else internal_date (tmp);	/* get current date in IMAP format */
 				/* add remainder of header */
-  sprintf (tmp+26,",%lu;%08lx%04x-00000000\015\012",size,uf,f);
+  sprintf (tmp+26,",%lu;%08lx%04lx-00000000\015\012",(unsigned long) size,uf,
+	   (unsigned long) f);
 				/* write header */
   if ((write (fd,tmp,strlen (tmp)) < 0) || ((write (fd,s,size)) < 0) ||
       fsync (fd)) {
@@ -941,7 +951,8 @@ long mbx_parse (MAILSTREAM *stream)
   short silent = stream->silent;
   fstat (LOCAL->fd,&sbuf);	/* get status */
   if (sbuf.st_size < curpos) {	/* sanity check */
-    sprintf (tmp,"Mailbox shrank from %lu to %lu!",curpos,sbuf.st_size);
+    sprintf (tmp,"Mailbox shrank from %lu to %lu!",
+	     (unsigned long) curpos,(unsigned long) sbuf.st_size);
     mm_log (tmp,ERROR);
     mbx_close (stream,NIL);
     return NIL;
@@ -956,7 +967,8 @@ long mbx_parse (MAILSTREAM *stream)
   stream->uid_validity = strtoul (LOCAL->buf + 7,NIL,16);
   LOCAL->buf[15] = c;		/* restore first character of last UID */
 				/* parse last UID */
-  stream->uid_last = strtoul (LOCAL->buf + 15,NIL,16);
+  i = strtoul (LOCAL->buf + 15,NIL,16);
+  stream->uid_last = stream->rdonly ? max (i,stream->uid_last) : i;
 				/* parse user flags */
   for (i = 0, s = LOCAL->buf + 25;
        (i < NUSERFLAGS) && (t = strchr (s,'\015')) && (t - s);
@@ -972,7 +984,8 @@ long mbx_parse (MAILSTREAM *stream)
     lseek (LOCAL->fd,curpos,L_SET);
     if ((i = read (LOCAL->fd,LOCAL->buf,64)) <= 0) {
       sprintf (tmp,"Unable to read internal header at %lu, size = %lu: %s",
-	       curpos,sbuf.st_size,i ? strerror (errno) : "no data read");
+	       (unsigned long) curpos,(unsigned long) sbuf.st_size,
+	       i ? strerror (errno) : "no data read");
       mm_log (tmp,ERROR);
       mbx_close (stream,NIL);
       return NIL;
@@ -980,7 +993,7 @@ long mbx_parse (MAILSTREAM *stream)
     LOCAL->buf[i] = '\0';	/* tie off buffer just in case */
     if (!((s = strchr (LOCAL->buf,'\015')) && (s[1] == '\012'))) {
       sprintf (tmp,"Unable to find CRLF at %lu in %lu bytes, text: %.80s",
-	       curpos,i,LOCAL->buf);
+	       (unsigned long) curpos,i,LOCAL->buf);
       mm_log (tmp,ERROR);
       mbx_close (stream,NIL);
       return NIL;
@@ -989,7 +1002,7 @@ long mbx_parse (MAILSTREAM *stream)
     i = (s + 2) - LOCAL->buf;	/* note start of text offset */
     if (!((s = strchr (LOCAL->buf,',')) && (t = strchr (s+1,';')))) {
       sprintf (tmp,"Unable to parse internal header at %lu: %.80s",
-	       curpos,LOCAL->buf);
+	       (unsigned long) curpos,LOCAL->buf);
       mm_log (tmp,ERROR);
       mbx_close (stream,NIL);
       return NIL;
@@ -999,7 +1012,7 @@ long mbx_parse (MAILSTREAM *stream)
 	  isxdigit (t[7]) && isxdigit (t[8]) && isxdigit (t[9]) &&
 	  isxdigit (t[10]) && isxdigit (t[11]) && isxdigit (t[12]))) {
       sprintf (tmp,"Unable to parse message flags at %lu: %.80s",
-	       curpos,LOCAL->buf);
+	       (unsigned long) curpos,LOCAL->buf);
       mm_log (tmp,ERROR);
       mbx_close (stream,NIL);
       return NIL;
@@ -1009,7 +1022,7 @@ long mbx_parse (MAILSTREAM *stream)
 	  isxdigit (t[17]) && isxdigit (t[18]) && isxdigit (t[19]) &&
 	  isxdigit (t[20]) && isxdigit (t[21]))) {
       sprintf (tmp,"Unable to parse message UID at %lu: %.80s",
-	       curpos,LOCAL->buf);
+	       (unsigned long) curpos,LOCAL->buf);
       mm_log (tmp,ERROR);
       mbx_close (stream,NIL);
       return NIL;
@@ -1019,7 +1032,7 @@ long mbx_parse (MAILSTREAM *stream)
 				/* get message size */
     if (!(j = strtoul (s,&x,10)) && (!(x && *x))) {
       sprintf (tmp,"Unable to parse message size at %lu: %.80s,%.80s;%.80s",
-	       curpos,LOCAL->buf,s,t);
+	       (unsigned long) curpos,LOCAL->buf,s,t);
       mm_log (tmp,ERROR);
       mbx_close (stream,NIL);
       return NIL;
@@ -1027,7 +1040,8 @@ long mbx_parse (MAILSTREAM *stream)
 				/* make sure didn't run off end of file */
     if (((off_t) (curpos + i + j)) > sbuf.st_size) {
       sprintf (tmp,"Last message (at %lu) runs past end of file (%lu > %lu)",
-	       curpos,curpos + i + j,sbuf.st_size);
+	       (unsigned long) curpos,(unsigned long) (curpos + i + j),
+	       (unsigned long) sbuf.st_size);
       mm_log (tmp,ERROR);
       mbx_close (stream,NIL);
       return NIL;
@@ -1035,7 +1049,7 @@ long mbx_parse (MAILSTREAM *stream)
 				/* parse UID */
     if ((m = strtoul (t+13,NIL,16)) &&
 	((m <= lastuid) || (m > stream->uid_last))) {
-      sprintf (tmp,"Invalid UID %08x in message %lu, rebuilding UIDs",
+      sprintf (tmp,"Invalid UID %08lx in message %lu, rebuilding UIDs",
 		 elt->private.uid,elt->msgno);
       mm_log (tmp,WARN);
       m = 0;			/* lose this UID */
@@ -1055,7 +1069,7 @@ long mbx_parse (MAILSTREAM *stream)
 				/* parse the date */
       if (!mail_parse_date (elt,LOCAL->buf)) {
 	sprintf (tmp,"Unable to parse message date at %lu: %.80s",
-		 curpos,LOCAL->buf);
+		 (unsigned long) curpos,LOCAL->buf);
 	mm_log (tmp,ERROR);
 	mbx_close (stream,NIL);
 	return NIL;
@@ -1230,10 +1244,10 @@ void mbx_update_status (MAILSTREAM *stream,unsigned long msgno,long flags)
       expflag = (strtoul (LOCAL->buf,NIL,16)) & fEXPUNGED;
     }
 				/* print new flag string */
-    sprintf (LOCAL->buf,"%08lx%04x-%08lx",elt->user_flags,
-	     expflag + (fSEEN * elt->seen) + (fDELETED * elt->deleted) +
-	     (fFLAGGED * elt->flagged) + (fANSWERED * elt->answered) +
-	     (fDRAFT * elt->draft),elt->private.uid);
+    sprintf (LOCAL->buf,"%08lx%04x-%08lx",elt->user_flags,(unsigned)
+	     (expflag + (fSEEN * elt->seen) + (fDELETED * elt->deleted) +
+	      (fFLAGGED * elt->flagged) + (fANSWERED * elt->answered) +
+	      (fDRAFT * elt->draft)),elt->private.uid);
     while (T) {			/* get to that place in the file */
       lseek (LOCAL->fd,(off_t) elt->private.special.offset +
 	     elt->private.special.text.size - 23,L_SET);
@@ -1271,7 +1285,7 @@ unsigned long mbx_hdrpos (MAILSTREAM *stream,unsigned long msgno,
   if (!(*size = elt->private.msg.header.text.size)) {
     lseek (LOCAL->fd,ret,L_SET);/* get to header position */
 				/* search message for CRLF CRLF */
-    for (siz = 1; siz <= elt->rfc822_size; siz++) {
+    for (siz = 1,s = tmp; siz <= elt->rfc822_size; siz++) {
 				/* read another buffer as necessary */
       if (--i <= 0)		/* buffer empty? */
 	if (read (LOCAL->fd,s = tmp,
