@@ -1,5 +1,5 @@
 /*
- * $Id: pico.h,v 4.106 1999/01/29 19:59:05 hubert Exp $
+ * $Id: pico.h,v 4.110 1999/06/10 03:35:15 mikes Exp $
  *
  * Program:	pico.h - definitions for Pine's composer library
  *
@@ -157,6 +157,14 @@ typedef struct pico_atmt {
     struct pico_atmt *next;
 } PATMT;
 
+/*
+ * Structure to contain color options
+ */
+typedef struct pico_colors {
+    COLOR_PAIR *tbcp;                   /* title bar color pair */
+    COLOR_PAIR *klcp;                   /* key label color pair */
+    COLOR_PAIR *kncp;                   /* key name color pair  */
+} PCOLORS;
 
 /*
  * Flags for attachment handling
@@ -185,6 +193,7 @@ typedef struct pico_struct {
     int    menu_rows;			/* number of rows in menu (0 or 2) */
     long   edit_offset;			/* offset into hdr line or body */
     PATMT *attachments;			/* linked list of attachments */
+    PCOLORS *colors;                    /* colors for titlebar and keymenu */
     long   pine_flags;			/* entry mode flags */
     int   (*helper)();			/* Pine's help function  */
     int   (*showmsg)();			/* Pine's display_message */
@@ -306,6 +315,8 @@ typedef struct menuitem {
     MPOINT	     lbl;	/* where the label starts		*/
     char	    *label;
     void            (*label_hiliter)();
+    COLOR_PAIR      *kncp;      /* key name color pair                  */
+    COLOR_PAIR      *klcp;      /* key label color pair                 */ 
     struct menuitem *next;
 } MENUITEM;
 #endif
@@ -392,7 +403,7 @@ extern	void	kbdestroy PROTO((KBESC_T *));
  */
 #ifdef	maindef
 PICO	*Pmaster = NULL;		/* composer specific stuff */
-char	*version = "3.5";		/* PICO version number */
+char	*version = "3.6";		/* PICO version number */
 
 #else
 extern	PICO *Pmaster;			/* composer specific stuff */
@@ -566,6 +577,9 @@ void  clear_mfunc PROTO((mousehandler_t));
 unsigned long mouse_in_content PROTO((int, int, int, int, int));
 unsigned long mouse_in_pico PROTO((int, int, int, int, int));
 void  mouse_get_last PROTO((mousehandler_t *, MOUSEPRESS *));
+void  register_key PROTO((int, unsigned, char *, void (*)(),
+			  int, int, int, COLOR_PAIR *, COLOR_PAIR *));
+int   mouse_on_key PROTO((int, int));
 int   checkmouse PROTO((unsigned *, int, int, int));
 void  invert_label PROTO((int, MENUITEM *));
 void  mouseon PROTO((void));
