@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: args.c,v 4.79 1999/01/07 22:01:22 mikes Exp $";
+static char rcsid[] = "$Id: args.c,v 4.83 1999/09/07 17:11:25 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -58,6 +58,8 @@ int  process_debug_str PROTO((char *));
 void args_add_attach PROTO((PATMT **, char *, int));
 void args_help PROTO((void));
 void display_args_err PROTO((char *, char **, int));
+#ifdef	_WINDOWS
+#endif
 
 
 /*
@@ -96,6 +98,7 @@ char *args_pine_args[] = {
 "\t\tNote: Attach options can't be used it -f, -F, or -url",
 "\t\tadded to Attachment list.  Attachlist must be the last",
 "\t\toption on the command line",
+" -bail\t\tExit if pinerc file doesn't already exist",
 #ifdef	DEBUG
 " -d n\t\tDebug - set debug level to 'n', or use the following:",
 " -d keywords...\tflush,timestamp,imap=0..4,numfiles=0..31,verbose=0..9",
@@ -118,6 +121,8 @@ char *args_pine_args[] = {
 " -p <pinerc>\tUse alternate .pinerc file",
 #if	!defined(DOS) && !defined(OS2)
 " -P <pine.conf>\tUse alternate pine.conf file",
+#else
+" -P <pine.conf>\tUse pine.conf file for default settings",
 #endif
 #if defined(OS2)
 " -w <rows>\tSet window size in rows on startup",
@@ -127,6 +132,9 @@ char *args_pine_args[] = {
 " -create_lu <abook_file> <ab_sort_type>   create .lu from script",
 " -nr\t\tSpecial mode for UWIN",
 " -a\t\tSpecial anonymous mode for UWIN",
+#if	defined(_WINDOWS)
+" -registry <cmd>\tcWhere cmd is set,clear,dump",
+#endif
 " -<option>=<value>   Assign <value> to the pinerc option <option>",
 "\t\t     e.g. -signature-file=sig1",
 "\t\t     (Note: feature-list is additive)",
@@ -344,6 +352,48 @@ Loop: while(--ac > 0)
 
 		  goto Loop;
 	      }
+	      else if(strcmp(*av, "bail") == 0){
+		  pine_state->exit_if_no_pinerc = 1;
+		  goto Loop;
+	      }
+#ifdef	_WINDOWS
+	      else if(strcmp(*av, "registry") == 0){
+		  if(--ac){
+		      if(!strucmp(*++av, "set")){
+			  pine_state->update_registry = 1;
+		      }
+		      else if(!strucmp(*av, "clear")){
+			  mswin_reg(MSWR_OP_BLAST, MSWR_NULL, NULL, 0);
+			  display_args_err(
+				      "Pine related Registry values removed.",
+				      NULL, 0);
+			  exit(0);
+		      }
+		      else if(!strucmp(*av, "dump")){
+#if	0
+			  char *pRegistry[] = mswin_reg_dump();
+
+			  if(pRegistry){
+			      display_args_err(NULL, pRegistry, 0);
+			      free_list_array(&pRegistry);
+			  }
+#endif
+		      }
+		      else{
+			  display_args_err("unknown registry command",
+					   NULL, 1);
+			  ++usage;
+		      }
+		  }
+		  else{
+		      sprintf(tmp_20k_buf, args_err_missing_flag_arg, c);
+		      display_args_err(tmp_20k_buf, NULL, 1);
+		      ++usage;
+		  }
+
+		  goto Loop;
+	      }
+#endif
 	      /* single char flags */
 	      else{
 		  switch(c = **av){
@@ -731,8 +781,10 @@ process_debug_str(debug_str)
 	}
 	else{
 	    debug = atoi(debug_str);
-	    if(debug > 8)
-	      ps_global->debug_imap = 4;
+	    if(debug > 8){
+		ps_global->debug_imap = 4;
+		ps_global->debug_timestamp = 1;
+	    }
 	    else if(debug > 6)
 	      ps_global->debug_imap = 3;
 	    else if(debug > 2)
