@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c,v 4.133 1998/12/29 20:08:48 hubert Exp $";
+static char rcsid[] = "$Id: imap.c,v 4.146 1999/10/11 22:11:42 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -81,6 +81,7 @@ long  imap_seq_exec PROTO((MAILSTREAM *, char *,
 			   long (*) PROTO((MAILSTREAM *, long, void *)),
 			   void *));
 long  imap_seq_exec_append PROTO((MAILSTREAM *, long, void *));
+char *imap_get_user PROTO((MMLOGIN_S *, STRLIST_S *));
 int   imap_get_passwd PROTO((MMLOGIN_S *, char *, char *, STRLIST_S *));
 void  imap_set_passwd PROTO((MMLOGIN_S **, char *, char *, STRLIST_S *));
 int   imap_same_host PROTO((STRLIST_S *, STRLIST_S *));
@@ -334,6 +335,7 @@ mm_exists(stream, number)
     unsigned long  number;
 {
     long new_this_call;
+    int	 exbits = 0;
 
 #ifdef DEBUG
     if(ps_global->debug_imap > 1)
@@ -343,28 +345,66 @@ mm_exists(stream, number)
 #endif
 
     if(stream == ps_global->mail_stream){
-	if(mn_get_total(ps_global->msgmap) != (long) number){
+	if(mn_get_nmsgs(ps_global->msgmap) != (long) number){
 	    ps_global->mail_box_changed = 1;
 	    ps_global->mangled_header	= 1;
 	}
 
-        if(mn_get_total(ps_global->msgmap) < (long) number){
-	    new_this_call = (long) number - mn_get_total(ps_global->msgmap);
+        if(mn_get_nmsgs(ps_global->msgmap) < (long) number){
+	    new_this_call = (long) number - mn_get_nmsgs(ps_global->msgmap);
 	    ps_global->new_mail_count += new_this_call;
 	    mn_add_raw(ps_global->msgmap, new_this_call);
+
+	    /*
+	     * Set local "recent" and "hidden" bits...
+	     */
+	    for(; new_this_call; --new_this_call, --number){
+		if(msgno_exceptions(stream, number, "0", &exbits, FALSE))
+		  exbits |= MSG_EX_RECENT;
+		else
+		  exbits = MSG_EX_RECENT;
+
+		msgno_exceptions(stream, number, "0", &exbits, TRUE);
+
+		/*
+		 * If we're zoomed, then hide this message too since
+		 * it couldn't have possibly been selected yet...
+		 */
+		if(any_lflagged(ps_global->msgmap, MN_HIDE))
+		  set_lflag(stream, ps_global->msgmap, number, MN_HIDE, 1);
+	    }
 	}
     } else if(stream == ps_global->inbox_stream) {
-	if(mn_get_total(ps_global->inbox_msgmap) != (long) number)
+	if(mn_get_nmsgs(ps_global->inbox_msgmap) != (long) number)
 	  ps_global->inbox_changed = 1;
 
-        if(mn_get_total(ps_global->inbox_msgmap) < (long) number){
-	    new_this_call = (long) number - mn_get_total(ps_global->inbox_msgmap);
+        if(mn_get_nmsgs(ps_global->inbox_msgmap) < (long) number){
+	    new_this_call = (long) number
+				      - mn_get_nmsgs(ps_global->inbox_msgmap);
 	    ps_global->inbox_new_mail_count += new_this_call;
 	    mn_add_raw(ps_global->inbox_msgmap, new_this_call);
+
+	    /*
+	     * Set local "recent" and "hidden" bits...
+	     */
+	    for(; new_this_call; --new_this_call, --number){
+		if(msgno_exceptions(stream, number, "0", &exbits, FALSE))
+		  exbits |= MSG_EX_RECENT;
+		else
+		  exbits = MSG_EX_RECENT;
+
+		msgno_exceptions(stream, number, "0", &exbits, TRUE);
+
+		/*
+		 * If we're zoomed, then hide this message too since
+		 * it couldn't have possibly been selected yet...
+		 */
+		if(any_lflagged(ps_global->inbox_msgmap, MN_HIDE))
+		  set_lflag(stream,ps_global->inbox_msgmap,number,MN_HIDE,1);
+	    }
 	}
-    } else {
-        /*--- ignore mm_exist for other. These are quick opens --*/
     }
+    /* else ignore streams we're not directly interested in */
 }
 
 
@@ -401,29 +441,9 @@ mm_expunged(stream, number)
      * If we ever deal with more than two streams, this'll break
      */
     if(stream == ps_global->mail_stream){
-	long i = mn_raw2m(ps_global->msgmap, (long) number);
-
-	if(i){
-	    /*
-	     * Keep on top of our special flag counts.
-	     * 
-	     * NOTE: This is allowed since mail_expunged releases
-	     * data for this message after the callback.
-	     */
-	    if(mc = mail_elt(stream, number)){
-		if(mc->spare)
-		  ps_global->msgmap->flagged_hid--;
-
-		if(mc->spare2)
-		  ps_global->msgmap->flagged_exld--;
-
-		if(mc->spare3)
-		  ps_global->msgmap->flagged_tmp--;
-
-		if(mc->sparep)
-		  msgno_free_exceptions((PARTEX_S **) &mc->sparep);
-	    }
+	long i;
 
+	if(i = mn_raw2m(ps_global->msgmap, (long) number)){
 	    /* flush invalid cache entries */
 	    while(i <= mn_get_total(ps_global->msgmap))
 	      clear_index_cache_ent(i++);
@@ -439,33 +459,60 @@ mm_expunged(stream, number)
 				 "Message you were viewing is gone!");
 	    }
 
-	    mn_flush_raw(ps_global->msgmap, (long) number);
 	    ps_global->mail_box_changed = 1;
 	    ps_global->mangled_header = 1;
 	    ps_global->expunge_count++;
 	}
+
+	/*
+	 * Keep on top of our special flag counts.
+	 * 
+	 * NOTE: This is allowed since mail_expunged releases
+	 * data for this message after the callback.
+	 */
+	if(mc = mail_elt(stream, number)){
+	    if(mc->spare)
+	      ps_global->msgmap->flagged_hid--;
+
+	    if(mc->spare2)
+	      ps_global->msgmap->flagged_exld--;
+
+	    if(mc->spare3)
+	      ps_global->msgmap->flagged_tmp--;
+
+	    if(mc->sparep)
+	      msgno_free_exceptions((PARTEX_S **) &mc->sparep);
+	}
+
+	/*
+	 * if it's in the sort array, flush it, otherwise
+	 * decrement raw sequence numbers greater than "number"
+	 */
+	mn_flush_raw(ps_global->msgmap, (long) number);
     }
     else if(stream == ps_global->inbox_stream){
-	long i = mn_raw2m(ps_global->inbox_msgmap, (long) number);
-	if(i){
-	    if(mc = mail_elt(stream, number)){
-		if(mc->spare)
-		  ps_global->inbox_msgmap->flagged_hid--;
+	long i;
 
-		if(mc->spare2)
-		  ps_global->inbox_msgmap->flagged_exld--;
-
-		if(mc->spare3)
-		  ps_global->inbox_msgmap->flagged_tmp--;
-
-		if(mc->sparep)
-		  msgno_free_exceptions((PARTEX_S **) &mc->sparep);
-	    }
-
-	    mn_flush_raw(ps_global->inbox_msgmap, (long) number);
+	if(i = mn_raw2m(ps_global->inbox_msgmap, (long) number)){
 	    ps_global->inbox_changed = 1;
 	    ps_global->inbox_expunge_count++;
 	}
+
+	mn_flush_raw(ps_global->inbox_msgmap, (long) number);
+
+	if(mc = mail_elt(stream, number)){
+	    if(mc->spare)
+	      ps_global->inbox_msgmap->flagged_hid--;
+
+	    if(mc->spare2)
+	      ps_global->inbox_msgmap->flagged_exld--;
+
+	    if(mc->spare3)
+	      ps_global->inbox_msgmap->flagged_tmp--;
+
+	    if(mc->sparep)
+	      msgno_free_exceptions((PARTEX_S **) &mc->sparep);
+	}
     }
 }
 
@@ -541,9 +588,9 @@ mm_login(mb, user, pwd, trial)
       hostlist[0].next = NULL;
 
     /*
-     * Initialize user name with either /user= value in the stream
-     * being logged into (accessed via the GET_USERNAMEBUF c-client
-     * parameter) or the user name we're running under...
+     * Initialize user name with either 
+     *     1) /user= value in the stream being logged into,
+     *  or 2) the user name we're running under.
      */
     if(trial == 0L){
 	strcpy(user, (*mb->user) ? mb->user : ps_global->VAR_USER_ID);
@@ -559,8 +606,22 @@ mm_login(mb, user, pwd, trial)
 	    return;
 	}
 #endif
+
+	/*
+	 * If no explicit user name supplied and we've not logged in
+	 * with our local user name, see if we've visited this
+	 * host before as someone else...
+	 */
+	if(!*mb->user && (last = imap_get_user(mm_login_list, hostlist)))
+	  strcpy(user, last);
     }
 
+    if(mb->altflag){
+	MoveCursor(0, ps_global->ttyo->screen_cols - 1);
+	StartInverse();
+	Write_to_screen("+");
+	EndInverse();
+    }
 
     ps_global->mangled_footer = 1;
     if(!*mb->user){
@@ -607,12 +668,22 @@ mm_login(mb, user, pwd, trial)
 	if(rc == 1 || !user[0]) {
 	    user[0]   = '\0';
 	    pwd[0] = '\0';
-	    return;
 	}
     }
     else
       strcpy(user, mb->user);
 
+    if(!user[0]){
+	if(mb->altflag){
+	    MoveCursor(0, ps_global->ttyo->screen_cols - 1);
+	    StartInverse();
+	    Write_to_screen(" ");
+	    EndInverse();
+	}
+
+	return;
+    }
+
     help = NO_HELP;
 
     /* Dress up long host/user names */
@@ -673,6 +744,13 @@ mm_login(mb, user, pwd, trial)
           break;
     }
 
+    if(mb->altflag){
+	MoveCursor(0, ps_global->ttyo->screen_cols - 1);
+	StartInverse();
+	Write_to_screen(" ");
+	EndInverse();
+    }
+
     if(rc == 1 || !pwd[0]) {
 	if(!*mb->user){
 	    strcpy(user, junk);
@@ -877,14 +955,15 @@ mm_flags(stream,number)
 	long i;
 
 	/* then clear index entry */
-	if(i = mn_raw2m(ps_global->msgmap, (long) number))
-	  clear_index_cache_ent(i);
+	if(i = mn_raw2m(ps_global->msgmap, (long) number)){
+	    clear_index_cache_ent(i);
 
-	/* in case number is current, fix titlebar */
-	if(mn_is_cur(ps_global->msgmap, i))
-	   ps_global->mangled_header = 1;
+	    /* in case number is current, fix titlebar */
+	    if(mn_is_cur(ps_global->msgmap, i))
+	      ps_global->mangled_header = 1;
 
-	check_point_change();
+	    check_point_change();
+	}
     }
 }
 
@@ -1058,25 +1137,39 @@ imap_referral(stream, ref, code)
     char *buf = NULL;
 
     if(ref && !struncmp(ref, "imap://", 7)){
-	char *folder;
+	char *folder = NULL;
 	long  uid_val, uid;
 	int rv;
 
 	rv = url_imap_folder(ref, &folder, &uid, &uid_val, NULL, 1);
-	switch(rv & URL_IMAP_MASK){
-	    /*
-	     * Only permit URLs specifying a simple "imessagelist" 
-	     */
-	  case URL_IMAP_IMESSAGELIST :
-	    buf = cpystr(folder);
+	switch(code){
+	  case REFAUTHFAILED :
+	  case REFAUTH :
+	    if((rv & URL_IMAP_IMAILBOXLIST) && (rv & URL_IMAP_ISERVERONLY))
+	      buf = cpystr(folder);
 
-	  case URL_IMAP_IMESSAGEPART :
-	  case URL_IMAP_IMAILBOXLIST :
-	    fs_give((void **) &folder);
+	    break;
+
+	  case REFSELECT :
+	  case REFCREATE :
+	  case REFDELETE :
+	  case REFRENAME :
+	  case REFSUBSCRIBE :
+	  case REFUNSUBSCRIBE :
+	  case REFSTATUS :
+	  case REFCOPY :
+	  case REFAPPEND :
+	    if(rv & URL_IMAP_IMESSAGELIST)
+	      buf = cpystr(folder);
+
+	    break;
 
 	  default :
 	    break;
 	}
+
+	if(folder)
+	  fs_give((void **) &folder);
     }
 
     return(buf);
@@ -1227,6 +1320,122 @@ imap_seq_exec_append(stream, msgno, args)
 }
 
 
+/*----------------------------------------------------------------------
+  This can be used to prevent the flickering of the check_cue char
+  caused by numerous (5000+) fetches by c-client.  Right now, the only
+  practical use found is newsgroup subsciption.
+
+  check_cue_display will check if this global is set, and won't clear
+  the check_cue_char if set.
+  ----*/
+void
+set_read_predicted(i)
+     int i;
+{
+    ps_global->read_predicted = i==1;
+#ifdef _WINDOWS
+    if(!i && F_ON(F_SHOW_DELAY_CUE, ps_global))
+      check_cue_display("  ");
+#endif
+}
+
+/*----------------------------------------------------------------------
+   Exported method to display status of mail check
+
+   Args: putstr -- should be NO LONGER THAN 2 bytes
+
+ Result: putstr displayed at upper-left-hand corner of screen
+  ----*/
+void
+check_cue_display(putstr)
+     char* putstr;
+{
+    COLOR_PAIR *lastc;
+    struct variable *vars = ps_global->vars;
+    static char check_cue_char;
+  
+    if(ps_global->read_predicted && 
+       (check_cue_char == putstr[0]
+	|| putstr[0] == ' ' && putstr[1] == '\0'))
+        return;
+    else{
+        if(putstr[0] == ' ')
+	  check_cue_char = '\0';
+	else
+	  check_cue_char = putstr[0];
+    }
+
+    lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
+			    PSC_REV|PSC_RET);
+    PutLine0(0, 0, putstr);		/* show delay cue */
+    if(lastc){
+	(void)pico_set_colorp(lastc, PSC_NONE);
+	free_color_pair(&lastc);
+    }
+
+    fflush(stdout);
+}
+
+/*----------------------------------------------------------------------
+   Exported method to retrieve logged in user name associated with stream
+
+   Args: host -- host to find associated login name with.
+
+ Result: 
+  ----*/
+void *
+pine_block_notify(reason, data)
+    int   reason;
+    void *data;
+{
+    static int mask = 0;
+    static int deep = 0;
+
+    switch(reason){
+      case BLOCK_SENSITIVE:		/* sensitive code, disallow alarms */
+	alrm_signal_block();
+	break;
+
+      case BLOCK_NONSENSITIVE:		/* non-sensitive code, allow alarms */
+	alrm_signal_unblock();
+	break;
+
+      case BLOCK_TCPWRITE:		/* blocked on TCP write */
+      case BLOCK_FILELOCK:		/* blocked on file locking */
+#ifdef	_WINDOWS
+	if(F_ON(F_SHOW_DELAY_CUE, ps_global))
+	  check_cue_display(">");
+
+	mswin_setcursor(MSWIN_CURSOR_BUSY);
+#endif
+	break;
+
+      case BLOCK_DNSLOOKUP:		/* blocked on DNS lookup */
+      case BLOCK_TCPOPEN:		/* blocked on TCP open */
+      case BLOCK_TCPREAD:		/* blocked on TCP read */
+      case BLOCK_TCPCLOSE:		/* blocked on TCP close */
+#ifdef	_WINDOWS
+	if(F_ON(F_SHOW_DELAY_CUE, ps_global))
+	  check_cue_display("<");
+
+	mswin_setcursor(MSWIN_CURSOR_BUSY);
+#endif
+	break;
+
+      default :
+      case BLOCK_NONE:			/* not blocked */
+#ifdef	_WINDOWS
+	if(F_ON(F_SHOW_DELAY_CUE, ps_global))
+	  check_cue_display(" ");
+#endif
+	break;
+
+    }
+
+    return(NULL);
+}
+
+
 
 /*----------------------------------------------------------------------
    Exported method to retrieve logged in user name associated with stream
@@ -1268,6 +1477,25 @@ imap_same_host(hl1, hl2)
 }
 
 
+char *
+imap_get_user(m_list, hostlist)
+    MMLOGIN_S *m_list;
+    STRLIST_S *hostlist;
+{
+    MMLOGIN_S *l;
+    STRLIST_S *h_list;
+    char      *h;
+    
+    for(l = m_list; l; l = l->next)
+      /* host name and user name must match */
+      if(imap_same_host(l->hosts, hostlist))
+	return(l->user);
+
+    return(NULL);
+}
+
+
+
 int
 imap_get_passwd(m_list, passwd, user, hostlist)
     MMLOGIN_S *m_list;
