@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: newmail.c,v 4.88 1999/01/27 20:38:12 hubert Exp $";
+static char rcsid[] = "$Id: newmail.c,v 4.102 1999/10/08 16:46:32 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -90,8 +90,9 @@ new_mail(force, time_for_check_point, flags)
     static time_t last_check = 0;
     static time_t last_check_point_call = 0;
     time_t        now;
-    long          rv = 0;
+    long          n, rv = 0;
     MAILSTREAM   *stream;
+    struct variable *vars = ps_global->vars;
     register struct pine *pine_state;
     int           checknow = 0;
 
@@ -112,7 +113,8 @@ new_mail(force, time_for_check_point, flags)
      * only check every 15 seconds, unless we're compelled to
      */
     if(!(stream = pine_state->mail_stream)
-       || (timeo == 0 && !force)
+       || !(timeo || force
+	    || pine_state->inbox_changed || pine_state->mail_box_changed)
        || (now-last_check_point_call <= 15 && time_for_check_point != 0
 	   && !pine_state->mail_box_changed && !force))
       return(-1);
@@ -129,12 +131,9 @@ new_mail(force, time_for_check_point, flags)
 			  ? BadTime : VeryBadTime, flags)
        && checknow){
 	if((flags & NM_STATUS_MSG) && F_ON(F_SHOW_DELAY_CUE, ps_global)
-	   && !ps_global->in_init_seq){
-	    StartInverse();
-	    PutLine0(0, 1, "*");	/* Show something to indicate delay */
+	   && !ps_global->in_init_seq)
+	    check_cue_display(" *");	/* Show something to indicate delay */
 	    MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
-	    fflush(stdout);
-	}
 #ifdef _WINDOWS
 	mswin_setcursor (MSWIN_CURSOR_BUSY);
 #endif
@@ -155,9 +154,7 @@ new_mail(force, time_for_check_point, flags)
 	if((flags & NM_STATUS_MSG)
 	   && F_ON(F_SHOW_DELAY_CUE, ps_global)
 	   && !ps_global->in_init_seq){
-	    PutLine0(0, 1, " ");
-	    EndInverse();
-	    fflush(stdout);
+	  check_cue_display("  ");
 	}
 #ifdef _WINDOWS
 	mswin_setcursor (MSWIN_CURSOR_ARROW);
@@ -167,12 +164,9 @@ new_mail(force, time_for_check_point, flags)
     if(checknow && pine_state->inbox_stream 
        && stream != pine_state->inbox_stream){
 	if((flags & NM_STATUS_MSG)
-	   && F_ON(F_SHOW_DELAY_CUE, ps_global)){
-	    StartInverse();
-	    PutLine0(0, 1, "*"); 	/* Show something to indicate delay */
-	    MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
-	    fflush(stdout);
-	}
+	   && F_ON(F_SHOW_DELAY_CUE, ps_global))
+	  check_cue_display(" *"); 	/* Show something to indicate delay */
+	  MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
 #ifdef _WINDOWS
 	mswin_setcursor (MSWIN_CURSOR_BUSY);
 #endif
@@ -189,17 +183,13 @@ new_mail(force, time_for_check_point, flags)
 	dprint(7, (debugfile, "Ping complete: %s\n", ctime(&now)));
 #endif
 	if((flags & NM_STATUS_MSG)
-	   && F_ON(F_SHOW_DELAY_CUE, ps_global)){
-	    PutLine0(0, 1, " ");
-	    EndInverse();
-	    fflush(stdout);
-	}
+	   && F_ON(F_SHOW_DELAY_CUE, ps_global))
+	    check_cue_display("  ");
 #ifdef _WINDOWS
 	mswin_setcursor (MSWIN_CURSOR_ARROW);
 #endif
     }
 
-
     /*-------------------------------------------------------------
        Mail box state changed, could be additions or deletions.
       -------------------------------------------------------------*/
@@ -216,6 +206,13 @@ new_mail(force, time_for_check_point, flags)
 	  fixup_flags(pine_state->mail_stream, pine_state->msgmap,
 		    pine_state->new_mail_count);
 
+	n = any_lflagged(pine_state->msgmap, MN_EXLD);
+	process_filter_patterns(pine_state->mail_stream,
+				pine_state->msgmap,
+				ps_global->new_mail_count);
+	if((n -= any_lflagged(pine_state->msgmap, MN_EXLD)) < 0)
+	  pine_state->new_mail_count += n;
+
 	/*
 	 * Lastly, worry about sorting if we got something new, otherwise
 	 * it was taken care of inside mm_expunge...
@@ -233,12 +230,17 @@ new_mail(force, time_for_check_point, flags)
 	    rv                         += pine_state->new_mail_count;
 	    ps_global->new_mail_count   = 0L;
 
-	    if(flags & NM_STATUS_MSG)
-	      new_mail_mess(pine_state->mail_stream,
+	    if(flags & NM_STATUS_MSG){
+		for(n = pine_state->mail_stream->nmsgs; n > 1L; n--)
+		  if(!get_lflag(pine_state->mail_stream, NULL, n, MN_EXLD))
+		    break;
+
+		if(n)
+		  new_mail_mess(pine_state->mail_stream,
 			 (pine_state->mail_stream == pine_state->inbox_stream)
 			    ? NULL :  pine_state->cur_folder,
-			 mailbox_mail_since_command,
-			 mn_get_total(pine_state->msgmap));
+			 mailbox_mail_since_command, n);
+	    }
         }
     }
 
@@ -255,15 +257,27 @@ new_mail(force, time_for_check_point, flags)
 	fixup_flags(pine_state->inbox_stream, pine_state->inbox_msgmap,
 		    pine_state->inbox_new_mail_count);
 
+	n = any_lflagged(pine_state->inbox_msgmap, MN_EXLD);
+	process_filter_patterns(pine_state->inbox_stream,
+				pine_state->inbox_msgmap,
+				pine_state->inbox_new_mail_count);
+	if((n -= any_lflagged(pine_state->inbox_msgmap, MN_EXLD)) < 0)
+	  pine_state->inbox_new_mail_count += n;
+
         if(pine_state->inbox_new_mail_count > 0) {
             inbox_mail_since_command       += pine_state->inbox_new_mail_count;
 	    rv                             += pine_state->inbox_new_mail_count;
             ps_global->inbox_new_mail_count = 0;
 
-	    if(flags & NM_STATUS_MSG)
-	      new_mail_mess(pine_state->inbox_stream, NULL,
-			    inbox_mail_since_command,
-			    mn_get_total(pine_state->inbox_msgmap));
+	    if(flags & NM_STATUS_MSG){
+		for(n = pine_state->inbox_stream->nmsgs; n > 1L; n--)
+		  if(!get_lflag(pine_state->inbox_stream, NULL, n, MN_EXLD))
+		    break;
+
+		if(n)
+		  new_mail_mess(pine_state->inbox_stream, NULL,
+			    inbox_mail_since_command, n);
+	    }
         }
     }
 
@@ -296,7 +310,6 @@ new_mail_mess(stream, folder, number, max_num)
      MAILSTREAM *stream;
      long        number, max_num;
      char       *folder;
-
 {
     ENVELOPE	*e;
     char	 subject[200], from[2*MAX_SCREEN_COLS],
@@ -317,20 +330,6 @@ new_mail_mess(stream, folder, number, max_num)
 
     e = mail_fetchstructure(stream, max_num, NULL);
 
-    if(!folder) {
-        if(number > 1)
-          sprintf(intro, "%ld new messages!", number);
-        else
-	  sprintf(intro, "New mail%s!",
-		  (e && address_is_us(e->to, ps_global)) ? " to you" : "");
-    }
-    else {
-        if(number > 1)
-          sprintf(intro,"%ld messages saved to folder \"%s\"", number, folder);
-        else
-          sprintf(intro, "Mail saved to folder \"%s\"", folder);
-    }
-
     if(e && e->from){
 	sprintf(from, " %srom ", (number > 1L) ? "Most recent f" : "F");
 	if(e->from->personal)
@@ -359,12 +358,41 @@ new_mail_mess(stream, folder, number, max_num)
 
         if(!from[0])
           subject[1] = toupper((unsigned char)subject[1]);
-
-        q_status_message3(SM_ASYNC | SM_DING, 0, 60,
-			  "%s%s%s", intro, from, subject);
     }
     else
-      q_status_message2(SM_ASYNC | SM_DING, 0, 60, "%s%s", intro, from);
+      subject[0] = '\0';
+
+    if(!folder) {
+        if(number > 1)
+          sprintf(intro, "%ld new messages!", number);
+        else
+	  sprintf(intro, "New mail%s!",
+		  (e && address_is_us(e->to, ps_global)) ? " to you" : "");
+    }
+    else {
+	long fl, tot, newfl;
+
+	if(number > 1)
+	  sprintf(intro, "%ld messages saved to folder \"%s\"", number,
+		  folder);
+	else
+	  sprintf(intro, "Mail saved to folder \"%s\"", folder);
+	
+	if((fl=strlen(folder)) > 10 &&
+	   (tot=strlen(intro) + strlen(from) + strlen(subject)) >
+					   ps_global->ttyo->screen_cols - 2){
+	    newfl = max(10, fl-(tot-(ps_global->ttyo->screen_cols - 2)));
+	    if(number > 1)
+	      sprintf(intro, "%ld messages saved to folder \"...%s\"", number,
+		      folder+(fl-(newfl-3)));
+	    else
+	      sprintf(intro, "Mail saved to folder \"...%s\"",
+		      folder+(fl-(newfl-3)));
+	}
+    }
+
+    q_status_message3(SM_ASYNC | SM_DING, 0, 60,
+		      "%s%s%s", intro, from, subject);
 
     sprintf(tmp_20k_buf, "%s%s", intro, from);
     icon_text(tmp_20k_buf);
@@ -391,19 +419,13 @@ fixup_flags(stream, msgmap, new_msgs)
     MSGNO_S    *msgmap;
     long	new_msgs;
 {
-    long i;
-
-    if(new_msgs > 0L && any_lflagged(msgmap, MN_HIDE)){
-	i = mn_get_total(msgmap) - new_msgs + 1L;
-	while(i > 0L && i <= mn_get_total(msgmap))
-	  set_lflag(stream, msgmap, i++, MN_HIDE, 1);
-    }
-
     /*
-     * Also, worry about the case where expunged away all of the 
+     * Deal with the case where expunged away all of the 
      * zoomed messages.  Unhide everything in that case...
      */
     if(mn_get_total(msgmap) > 0L){
+	long i;
+
 	if(any_lflagged(msgmap, MN_HIDE) >= mn_get_total(msgmap)){
 	    for(i = 1L; i <= mn_get_total(msgmap); i++)
 	      set_lflag(stream, msgmap, i, MN_HIDE, 0);
@@ -462,6 +484,8 @@ check_point(timing, flags)
     int     freq, tm;
     long    adj_cca;
     long    tmp;
+    COLOR_PAIR *lastc = NULL, *newc;
+    struct variable *vars = ps_global->vars;
 #ifdef	DEBUG
     time_t  now;
 #endif
@@ -507,10 +531,8 @@ check_point(timing, flags)
                      ctime(&now), now - first_status_change));
 #endif
 	if((flags & NM_STATUS_MSG) && F_ON(F_SHOW_DELAY_CUE, ps_global)){
-	    StartInverse();
-	    PutLine0(0, 0, "**");	/* Show something indicate delay*/
+	    check_cue_display("**");	/* Show something indicate delay*/
 	    MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
-	    fflush(stdout);
 	}
 #ifdef _WINDOWS
 	mswin_setcursor (MSWIN_CURSOR_BUSY);
@@ -524,11 +546,8 @@ check_point(timing, flags)
 #endif
         check_count = 0;
         first_status_change = time(0);
-	if((flags & NM_STATUS_MSG) && F_ON(F_SHOW_DELAY_CUE, ps_global)){
-	    PutLine0(0, 0, "  ");
-	    EndInverse();
-	    fflush(stdout);
-	}
+	if((flags & NM_STATUS_MSG) && F_ON(F_SHOW_DELAY_CUE, ps_global))
+	    check_cue_display("  ");
 #ifdef _WINDOWS
 	mswin_setcursor (MSWIN_CURSOR_ARROW);
 #endif
