@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: signals.c,v 4.127 1998/11/12 22:48:57 hubert Exp $";
+static char rcsid[] = "$Id: signals.c,v 4.129 1999/04/21 00:24:37 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -61,7 +61,6 @@ void  fast_clean_up PROTO(());
 void  suspend_notice PROTO((char *));
 void  suspend_warning PROTO(());
 void  alarm_signal_reset PROTO(());
-void  alarm_wrapper PROTO ((unsigned));
 
 
               /* SigType is defined in os.h and is either int or void */
@@ -608,19 +607,11 @@ busy_alarm(seconds, msg, pc_func, init_msg)
 
     /* set alarm */
     if(F_OFF(F_DISABLE_ALARM, ps_global))
-      alarm_wrapper(seconds);
+      alarm(seconds);
 
     return(retval);
 }
 
-void
-alarm_wrapper(seconds)
-    unsigned seconds;
-{
-    mail_parameters(NULL, SET_ALARMSAVE, (void *) (seconds ? 1L : 0L));
-    (void)alarm(seconds);
-}
-
 
 /*
  * If final_message was set when busy_alarm was called:
@@ -633,7 +624,7 @@ cancel_busy_alarm(message_pri)
 {
     dprint(9,(debugfile, "cancel_busy_alarm(%d)\n", message_pri));
 
-    alarm_wrapper(0);
+    alarm(0);
 
     if(busy_alarm_outstanding){
 	int space_left, slots_used;
@@ -681,7 +672,7 @@ suspend_busy_alarm()
     dprint(9,(debugfile, "suspend_busy_alarm\n"));
 
     if(busy_alarm_outstanding)
-      alarm_wrapper(0);
+      alarm(0);
 }
 
 
@@ -695,7 +686,46 @@ resume_busy_alarm(pause)
     dprint(9,(debugfile, "resume_busy_alarm\n"));
 
     if(busy_alarm_outstanding && F_OFF(F_DISABLE_ALARM, ps_global))
-      alarm_wrapper(pause ? pause : alarm_increment);
+      alarm(pause ? pause : alarm_increment);
+}
+
+
+/*
+ * Block our alarm signals when we enter sensitive code, and unblock when
+ * we leave. These are called from a c-client callback.
+ */
+#ifdef	sigmask
+static int alrm_depth = 0;
+static int alrm_mask = 0;
+#endif
+
+void
+alrm_signal_block()
+{
+#ifdef	sigmask
+    if(!alrm_depth++)
+      alrm_mask = sigblock(sigmask(SIGALRM));
+#endif
+}
+
+
+void
+alrm_signal_unblock()
+{
+#ifdef	sigmask
+    switch(alrm_depth){
+      case 1:			/* pop */
+	sigsetmask(alrm_mask);
+	alrm_depth = 0;
+
+      case 0:			/* unbalanced unblocks */
+	break;
+
+      default:			/* nested unblock */
+	alrm_depth--;
+	break;
+    }
+#endif
 }
 
 
@@ -797,7 +827,7 @@ alarm_signal_reset()
 #if	!defined(DOS) || defined(_WINDOWS)
     signal(SIGALRM, alarm_signal);
 #endif
-    alarm_wrapper(alarm_increment);
+    alarm(alarm_increment);
 }
 
 
@@ -830,16 +860,6 @@ static SigType
 intr_signal SIG_PROTO((int sig))
 {
     ps_global->intr_pending = 1;
-
-    /*
-     * Anyone else want to know?
-     */
-    {
-	extern SORTPGM *g_sort_prog;
-
-	if(g_sort_prog)
-	  g_sort_prog->abort = 1;
-    }
 }
 
 
