@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: status.c,v 4.107 1998/08/20 07:40:22 mikes Exp $";
+static char rcsid[] = "$Id: status.c,v 4.116 1999/07/07 23:40:19 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: status.c,v 4.107 1998/08/20 07:40:22 mikes Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1998 by the University of Washington.
+   1989-1999 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -137,15 +137,19 @@ q_status_message(flags, min_time, max_time, message)
 {
     SMQ_T *new;
 
-    if(message_queue){
-	if(!strcmp(message_queue->text, message)){
-	    message_queue->shown = 0;
+    /* Hunt to last message -- if same already queued, move on... */
+    if(new = message_queue){
+	while(new->next != message_queue)
+	  new = new->next;
 
-	    if(message_queue->min_display_time < min_time)
-	      message_queue->min_display_time = min_time;
+	if(!strcmp(new->text, message)){
+	    new->shown = 0;
 
-	    if(message_queue->max_display_time < max_time)
-	      message_queue->max_display_time = max_time;
+	    if(new->min_display_time < min_time)
+	      new->min_display_time = min_time;
+
+	    if(new->max_display_time < max_time)
+	      new->max_display_time = max_time;
 
 	    return;
 	}
@@ -519,8 +523,25 @@ display_message(command)
     }
 
     if(!message_queue && (command == ctrl('L') || needs_clearing)) {
+	int inverse;
+	struct variable *vars = ps_global->vars;
+	char *last_bg = NULL;
+
 	dprint(9, (debugfile, "Clearing status line\n"));
+	inverse = InverseState();	/* already in inverse? */
+	if(inverse && pico_usingcolor() && VAR_STATUS_FORE_COLOR &&
+	   VAR_STATUS_BACK_COLOR){
+	    last_bg = pico_get_last_bg_color();
+	    pico_set_nbg_color();   /* so ClearLine will clear in bg color */
+	}
+
 	ClearLine(ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global));
+	if(last_bg){
+	    (void)pico_set_bg_color(last_bg);
+	    if(last_bg)
+	      fs_give((void **)&last_bg);
+	}
+
 	mark_status_unknown();
 	if(command == ctrl('L')){
 	    mark_keymenu_dirty();
@@ -655,6 +676,8 @@ status_message_write(message, from_alarm_handler)
 {
     int  col, row, max_length, invert;
     char obuff[MAX_SCREEN_COLS + 1];
+    struct variable *vars = ps_global->vars;
+    COLOR_PAIR *lastc = NULL, *newc;
 
     if(!from_alarm_handler)
       add_review_message(message);
@@ -682,7 +705,19 @@ status_message_write(message, from_alarm_handler)
 	    char *p, *q, *uneq_str;
 	    int   column, start_col;
 
-	    if(invert)
+	    if(pico_usingcolor() && VAR_STATUS_FORE_COLOR &&
+	       VAR_STATUS_BACK_COLOR &&
+	       pico_is_good_color(VAR_STATUS_FORE_COLOR) &&
+	       pico_is_good_color(VAR_STATUS_BACK_COLOR)){
+		lastc = pico_get_cur_color();
+		if(lastc){
+		    newc = new_color_pair(VAR_STATUS_FORE_COLOR,
+					  VAR_STATUS_BACK_COLOR);
+		    (void)pico_set_colorp(newc, PSC_NONE);
+		    free_color_pair(&newc);
+		}
+	    }
+	    else if(invert)
 	      StartInverse();
 
 	    q = prevstatusbuff;
@@ -720,7 +755,11 @@ status_message_write(message, from_alarm_handler)
 		}
 	    }
 
-	    if(invert)
+	    if(lastc){
+		(void)pico_set_colorp(lastc, PSC_NONE);
+		free_color_pair(&lastc);
+	    }
+	    else if(invert)
 	      EndInverse();
 
 	    /* move cursor to a consistent position */
@@ -728,12 +767,42 @@ status_message_write(message, from_alarm_handler)
 	    fflush(stdout);
 	}
 	else{
+	    if(pico_usingcolor())
+	      lastc = pico_get_cur_color();
+
+	    if(!invert && pico_usingcolor() && VAR_STATUS_FORE_COLOR &&
+	       VAR_STATUS_BACK_COLOR &&
+	       pico_is_good_color(VAR_STATUS_FORE_COLOR) &&
+	       pico_is_good_color(VAR_STATUS_BACK_COLOR))
+	      pico_set_nbg_color();	/* so ClearLine uses bg color */
+
 	    ClearLine(row);
-	    if(invert)
-	      StartInverse();
+
+	    if(pico_usingcolor() && VAR_STATUS_FORE_COLOR &&
+	       VAR_STATUS_BACK_COLOR &&
+	       pico_is_good_color(VAR_STATUS_FORE_COLOR) &&
+	       pico_is_good_color(VAR_STATUS_BACK_COLOR)){
+		if(lastc){
+		    newc = new_color_pair(VAR_STATUS_FORE_COLOR,
+					  VAR_STATUS_BACK_COLOR);
+		    (void)pico_set_colorp(newc, PSC_NONE);
+		    free_color_pair(&newc);
+		}
+	    }
+	    else if(invert){
+		if(lastc)
+		  free_color_pair(&lastc);
+
+		StartInverse();
+	    }
+
 
 	    col = Centerline(row, obuff);
-	    if(invert)
+	    if(lastc){
+		(void)pico_set_colorp(lastc, PSC_NONE);
+		free_color_pair(&lastc);
+	    }
+	    else if(invert)
 	      EndInverse();
 
 	    MoveCursor(row, 0);
@@ -898,18 +967,27 @@ delay_cmd_cue(on)
     int on;
 {
     int l;
+    COLOR_PAIR *lastc;
+    struct variable *vars = ps_global->vars;
 
     if(prevstartcol >= 0 && (l = strlen(prevstatusbuff))){
 	MoveCursor(ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global),
 		   prevstartcol ? max(prevstartcol - 1, 0) : 0);
-	StartInverse();
+	lastc = pico_set_colors(VAR_STATUS_FORE_COLOR, VAR_STATUS_BACK_COLOR,
+				PSC_REV|PSC_RET);
+
 	Write_to_screen(on ? "[>" : " [");
-	EndInverse();
+
 	MoveCursor(ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global),
 		   min(prevstartcol + l, ps_global->ttyo->screen_cols) - 1);
-	StartInverse();
+
 	Write_to_screen(on ? "<]" : "] ");
-	EndInverse();
+
+	if(lastc){
+	    (void)pico_set_colorp(lastc, PSC_NONE);
+	    free_color_pair(&lastc);
+	}
+
 	MoveCursor(ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global), 0);
     }
 
@@ -1081,6 +1159,8 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
     struct key	     rb_keys[12];
     struct key_menu  rb_keymenu;
     bitmap_t	     bitmap;
+    struct variable *vars = ps_global->vars;
+    COLOR_PAIR      *lastc = NULL, *promptc = NULL;
 #ifdef	_WINDOWS
     int		     cursor_shown;
 #endif
@@ -1227,19 +1307,41 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
     ps_global->mangled_footer = 1;
 
 #ifdef	_WINDOWS
-    cursor_shown = mswin_showcursor(1);
+    cursor_shown = mswin_showcaret(1);
 #endif
 
+    if(pico_usingcolor() && VAR_PROMPT_FORE_COLOR &&
+       VAR_PROMPT_BACK_COLOR &&
+       pico_is_good_color(VAR_PROMPT_FORE_COLOR) &&
+       pico_is_good_color(VAR_PROMPT_BACK_COLOR)){
+	lastc = pico_get_cur_color();
+	if(lastc){
+	    promptc = new_color_pair(VAR_PROMPT_FORE_COLOR,
+				     VAR_PROMPT_BACK_COLOR);
+	    (void)pico_set_colorp(promptc, PSC_NONE);
+	}
+    }
+    else
+      StartInverse();
+
     draw_radio_prompt(real_line, RAD_BUT_COL, maxcol, q);
 
     while(1){
         fflush(stdout);
 
 	/*---- Paint the keymenu ----*/
-	EndInverse();
+	if(lastc)
+	  (void)pico_set_colorp(lastc, PSC_NONE);
+	else
+	  EndInverse();
+
 	draw_keymenu(&rb_keymenu, bitmap, ps_global->ttyo->screen_cols,
 		     1 - FOOTER_ROWS(ps_global), 0, FirstMenu);
-	StartInverse();
+	if(promptc)
+	  (void)pico_set_colorp(promptc, PSC_NONE);
+	else
+	  StartInverse();
+
 	MoveCursor(real_line, min(RAD_BUT_COL+strlen(q), maxcol+1));
 
 	if(flags & RB_FLUSH_IN)
@@ -1318,13 +1420,28 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 		FOOTER_ROWS(ps_global) = 3;
 		line = -3;
 		real_line = ps_global->ttyo->screen_rows + line;
+		if(lastc)
+		  (void)pico_set_colorp(lastc, PSC_NONE);
+		else
+		  EndInverse();
+
 		clearfooter(ps_global);
+		if(promptc)
+		  (void)pico_set_colorp(promptc, PSC_NONE);
+		else
+		  StartInverse();
+
 		draw_radio_prompt(real_line, RAD_BUT_COL, maxcol, q);
 		break;
 	    }
 
 	    if(help_text != NO_HELP && FOOTER_ROWS(ps_global) > 1){
 		mark_keymenu_dirty();
+		if(lastc)
+		  (void)pico_set_colorp(lastc, PSC_NONE);
+		else
+		  EndInverse();
+
 		MoveCursor(real_line + 1, RAD_BUT_COL);
 		CleartoEOLN();
 		MoveCursor(real_line + 2, RAD_BUT_COL);
@@ -1332,6 +1449,10 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 		radio_help(real_line, RAD_BUT_COL, help_text);
 		sleep(5);
 		MoveCursor(real_line, min(RAD_BUT_COL+strlen(q), maxcol+1));
+		if(promptc)
+		  (void)pico_set_colorp(promptc, PSC_NONE);
+		else
+		  StartInverse();
 	    }
 	    else
 	      Writechar(BELL, 0);
@@ -1364,7 +1485,11 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
           case KEY_RESIZE:
           case ctrl('L'):
             real_line = line > 0 ? line : ps_global->ttyo->screen_rows + line;
-            EndInverse();
+	    if(lastc)
+	      (void)pico_set_colorp(lastc, PSC_NONE);
+	    else
+	      EndInverse();
+
             ClearScreen();
             redraw_titlebar();
             if(ps_global->redrawer != NULL)
@@ -1373,6 +1498,11 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
               redraw_keymenu();
 
 	    maxcol = ps_global->ttyo->screen_cols - max_label - 1;
+	    if(promptc)
+	      (void)pico_set_colorp(promptc, PSC_NONE);
+	    else
+	      StartInverse();
+
 	    draw_radio_prompt(real_line, RAD_BUT_COL, maxcol, q);
             break;
 
@@ -1383,14 +1513,22 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 
 #ifdef	_WINDOWS
     if(!cursor_shown)
-      mswin_showcursor(0);
+      mswin_showcaret(0);
 #endif
 
     fs_give((void **)&q);
     if(ds)
       fs_give((void **)&ds);
 
-    EndInverse();
+    if(lastc){
+	(void)pico_set_colorp(lastc, PSC_NONE);
+	free_color_pair(&lastc);
+	if(promptc)
+	  free_color_pair(&promptc);
+    }
+    else
+      EndInverse();
+
     fflush(stdout);
     resume_busy_alarm(0);
     if(km_popped){
@@ -1421,7 +1559,6 @@ radio_help(line, column, help)
     if(text == NULL)
       return;
     
-    EndInverse();
     MoveCursor(line + 1, column);
     CleartoEOLN();
     if(text[0])
@@ -1432,7 +1569,6 @@ radio_help(line, column, help)
     if(text[1])
       PutLine0(line + 2, column, text[1]);
 
-    StartInverse();
 #if defined(HELPFILE)
     free_list_array(&text);
 #endif
@@ -1457,7 +1593,6 @@ draw_radio_prompt(line, start_c, max_c, q)
 	len = max_c - start_c + 1;
     }
 
-    StartInverse();
     PutLine0(line, start_c, q);
     x = start_c + len;
     MoveCursor(line, x);
